@@ -91,6 +91,35 @@ void Document_Glom::set_connection_database(const Glib::ustring& strVal)
   }
 }
 
+void Document_Glom::set_relationship(const Glib::ustring& table_name, const Relationship& relationship)
+{
+  //Find the existing relationship:
+  type_tables::iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    DocumentTableInfo& info = iterFind->second;
+    
+    //Look for the relationship with this name:
+    bool existing = false;
+    const Glib::ustring relationship_name = relationship.get_name();
+  
+    for(type_vecRelationships::iterator iter = info.m_relationships.begin(); iter != info.m_relationships.end(); ++iter)
+    {
+      if(iter->get_name() == relationship_name)
+      {
+        *iter = relationship;
+        existing = true;
+      }        
+    }
+    
+    if(!existing)
+    {
+      //Add a new one if it's not there.
+      info.m_relationships.push_back(relationship);
+    }
+  }
+  
+}
 
 bool Document_Glom::get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const
 {
@@ -143,7 +172,14 @@ Document_Glom::type_vecFields Document_Glom::get_table_fields(const Glib::ustrin
   {
     type_tables::const_iterator iterFind = m_tables.find(table_name);
     if(iterFind != m_tables.end())
+    {
+      if(iterFind->second.m_fields.empty())
+      {
+         g_warning("Document_Glom::get_table_fields: table found, but m_fields is empty");
+      }
+      
       return iterFind->second.m_fields;
+    }
     else
     {
       g_warning("Document_Glom::get_table_fields: table not found in document: %s", table_name.c_str());
@@ -160,7 +196,12 @@ Document_Glom::type_vecFields Document_Glom::get_table_fields(const Glib::ustrin
 void Document_Glom::set_table_fields(const Glib::ustring& table_name, const type_vecFields& vecFields)
 {
   if(!table_name.empty())
-  { 
+  {
+    if(vecFields.empty())
+    {
+      g_warning("Document_Glom::set_table_fields(): vecFields is empty: table_name=%s", table_name.c_str());
+    }
+      
     DocumentTableInfo& info = get_table_info_with_add(table_name);
     info.m_fields = vecFields;
 
@@ -200,7 +241,7 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
       iterFind->set_name(strFieldNameNew);
     }
     
-    //Find any relationships or layouts that use this fieldt
+    //Find any relationships or layouts that use this field
     //Look at each table:
     for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
     {
@@ -370,6 +411,12 @@ void Document_Glom::set_tables(const type_listTableInfo& tables)
   
 }
 
+Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_relationship_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Relationship& relationship) const
+{
+  //TODO: Use an actual relationship_name instead of concatenating:
+  return get_data_layout_groups_plus_new_fields(layout_name, relationship.get_from_table() + "_related_" + relationship.get_name()); 
+}
+
 Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& table_name) const
 {
   type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, table_name);
@@ -407,27 +454,30 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
     for(type_vecFields::const_iterator iter = all_fields.begin(); iter != all_fields.end(); ++iter)
     {
       const Glib::ustring field_name = iter->get_name();
-
-      //See whether it's already in the result:
-      //TODO_Performance: There is a lot of iterating and comparison here:
-      bool found = false; //TODO: This is horrible.
-      for(type_mapLayoutGroupSequence::const_iterator iterFind = result.begin(); iterFind != result.end(); ++iterFind)
+      if(!field_name.empty())
       {
-        if(iterFind->second.has_field(field_name))
+        //See whether it's already in the result:
+        //TODO_Performance: There is a lot of iterating and comparison here:
+        bool found = false; //TODO: This is horrible.
+        for(type_mapLayoutGroupSequence::const_iterator iterFind = result.begin(); iterFind != result.end(); ++iterFind)
         {
-          found = true;
-          break;
+          if(iterFind->second.has_field(field_name))
+          {
+            found = true;
+            break;
+          }
+        }
+  
+        if(!found)
+        {
+          LayoutItem_Field layout_item;
+          layout_item.set_name(field_name);
+          layout_item.set_table_name(table_name); //TODO: Allow viewing of fields through relationships.
+          //layout_item.m_sequence = sequence;  add_item() will fill this.
+          layout_item.m_hidden = false;
+  
+          pTopLevel->add_item(layout_item);
         }
-      }
-
-      if(!found)
-      {
-        LayoutItem_Field layout_item;
-        layout_item.set_name(field_name);
-        //layout_item.m_sequence = sequence;  add_item() will fill this.
-        layout_item.m_hidden = false;
-
-        pTopLevel->add_item(layout_item);
       }
     }
   }
@@ -453,6 +503,12 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   return type_mapLayoutGroupSequence(); //not found
 }
 
+void Document_Glom::set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups)
+{
+  //TODO: Use an actual relationship_name instead of concatenating:
+  set_data_layout_groups(layout_name, relationship.get_from_table() + "_related_" + relationship.get_name(), groups);
+}
+
 void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name, const type_mapLayoutGroupSequence& groups)
 {
   if(!table_name.empty())
@@ -508,8 +564,18 @@ bool Document_Glom::get_table_is_known(const Glib::ustring& table_name) const
 
 AppState::userlevels Document_Glom::get_userlevel() const
 {
+  userLevelReason reason;
+  return get_userlevel(reason);
+}
+
+AppState::userlevels Document_Glom::get_userlevel(userLevelReason& reason) const
+{
+  //Initialize output parameter:
+  reason = USER_LEVEL_REASON_UNKNOWN;
+  
   if(get_read_only())
   {
+    reason = USER_LEVEL_REASON_FILE_READ_ONLY;
     return AppState::USERLEVEL_OPERATOR; //A read-only document can not be changed, so there's no point in being in developer mode. This is one way to control the user level on purpose.
   }
   else if(m_file_uri.empty()) //If it has never been saved then this is a new default document, so the user created it, so the user can be a developer.
@@ -532,9 +598,19 @@ void Document_Glom::on_app_state_userlevel_changed(AppState::userlevels userleve
   m_signal_userlevel_changed.emit(userlevel);
 }
 
-void Document_Glom::set_userlevel(AppState::userlevels userlevel)
+bool Document_Glom::set_userlevel(AppState::userlevels userlevel)
 {
-  m_app_state.set_userlevel(userlevel);
+  //Prevent incorrect user level:
+  if((userlevel == AppState::USERLEVEL_DEVELOPER) && get_read_only())
+  {
+    m_app_state.set_userlevel(AppState::USERLEVEL_OPERATOR);
+    return false;
+  }
+  else
+  { 
+    m_app_state.set_userlevel(userlevel);
+    return true;
+  }
 }
 
 void Document_Glom::emit_userlevel_changed()
@@ -589,8 +665,11 @@ void Document_Glom::set_modified(bool value)
   }
 }
 
-void Document_Glom::load_after_layout_group(const xmlpp::Element* node, LayoutGroup& group)
+void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, LayoutGroup& group)
 {
+  if(!node)
+    return;
+    
   //Get the group details:
   group.set_name( get_node_attribute_value(node, "name") );
   group.m_title = get_node_attribute_value(node, "title");
@@ -603,24 +682,36 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, LayoutGr
   for(xmlpp::Node::NodeList::iterator iter = listNodes.begin(); iter != listNodes.end(); ++iter)
   {
     const xmlpp::Element* element = dynamic_cast<const xmlpp::Element*>(*iter);
-
-    if(element->get_name() == "data_layout_item")
+    if(element)
     {
-      LayoutItem_Field item;
-
-      item.set_name( get_node_attribute_value(element, "name") );
-
       const guint sequence = get_node_attribute_value_as_decimal(element, "sequence");
-      item.m_sequence = sequence;
-      group.add_item(item, sequence);
-    }
-    else if(element->get_name() == "data_layout_group")
-    {
-      LayoutGroup child_group;
-      //Recurse:
-      load_after_layout_group(element, child_group);
+      
+      if(element->get_name() == "data_layout_item")
+      {
+        LayoutItem_Field item;
 
-      group.add_item(child_group);
+        item.set_name( get_node_attribute_value(element, "name") );
+        item.set_table_name(table_name);
+
+        item.m_sequence = sequence;
+        group.add_item(item, sequence);
+      }
+      else if(element->get_name() == "data_layout_group")
+      {
+        LayoutGroup child_group;
+        //Recurse:
+        load_after_layout_group(element, table_name, child_group);
+
+        group.add_item(child_group);
+      }
+      else if(element->get_name() == "data_layout_portal")
+      {
+        LayoutItem_Portal item;
+        item.set_relationship( get_node_attribute_value(element, "relationship") );
+
+        item.m_sequence = sequence;
+        group.add_item(item, sequence);
+      }
     }
   }                          
 }
@@ -707,7 +798,7 @@ bool Document_Glom::load_after()
                 
                 //Field Type:
                 const Glib::ustring field_type = get_node_attribute_value(nodeChild, "type");
-                
+      
                 //Get the type enum for this string representation of the type:
                 Field::glom_field_type field_type_enum = Field::TYPE_INVALID;
                 for(Field::type_map_type_names::const_iterator iter = type_names.begin(); iter !=type_names.end(); ++iter)
@@ -718,15 +809,16 @@ bool Document_Glom::load_after()
                     break;
                   }
                 }
-                
-                field.set_glom_type( field_type_enum );
-
+                        
                 //Default value:
                 const Glib::ustring default_value_text = get_node_attribute_value(nodeChild, "default_value");
                 //Interpret the text as per the field type:
                 bool success = false;
                 field_info.set_default_value( GlomConversions::parse_value(field_type_enum, default_value_text, success, true /* iso_format */) );
                 field.set_field_info(field_info);
+
+                //We set this after set_field_info(), because that gets a glom type from the (not-specified) gdatype. Yes, that's strange, and should probably be more explicit.
+                field.set_glom_type( field_type_enum );
                              
                 doctableinfo.m_fields.push_back(field);
               }
@@ -748,6 +840,10 @@ bool Document_Glom::load_after()
                 
                 relationship.set_from_table( table_name );
                 relationship.set_name( relationship_name );
+                
+                const Glib::ustring relationship_title = get_node_attribute_value(nodeChild, "title");
+                relationship.set_title( relationship_title );
+                
                 relationship.set_from_field( get_node_attribute_value(nodeChild, "key") );
                 relationship.set_to_table( get_node_attribute_value(nodeChild, "other_table") );
                 relationship.set_to_field( get_node_attribute_value(nodeChild, "other_key") );
@@ -785,7 +881,7 @@ bool Document_Glom::load_after()
                       if(!group_name.empty())
                       {
                         LayoutGroup group;
-                        load_after_layout_group(node, group);
+                        load_after_layout_group(node, table_name, group);
 
                         layout_groups[group.m_sequence] = group;
                       }
@@ -837,6 +933,18 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
 
         set_node_attribute_value_as_decimal(nodeItem, "sequence", item->m_sequence);
       }
+      else
+      {
+        const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
+        if(portal) //If it is a portal
+        {
+          xmlpp::Element* nodeItem = child->add_child("data_layout_portal");
+          nodeItem->set_attribute("relationship", portal->get_relationship());
+
+          set_node_attribute_value_as_decimal(nodeItem, "sequence", item->m_sequence);
+        }
+      }
+        
     }
   }        
 }
@@ -922,6 +1030,7 @@ bool Document_Glom::save_before()
 
           xmlpp::Element* elemRelationship = elemRelationships->add_child("relationship");
           set_node_attribute_value(elemRelationship, "name", relationship.get_name());
+          set_node_attribute_value(elemRelationship, "title", relationship.get_title());
           set_node_attribute_value(elemRelationship, "key", relationship.get_from_field());
           set_node_attribute_value(elemRelationship, "other_table", relationship.get_to_table());
           set_node_attribute_value(elemRelationship, "other_key", relationship.get_to_field());
