@@ -105,7 +105,9 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
   show_all();
-   
+
+  m_FlowTable.signal_layout_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_layout_changed) );
+
   m_ignore_signals = false;
 }
 
@@ -122,6 +124,8 @@ void Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const
 {
   m_primary_key_value = primary_key_value;
 
+  get_field_primary_key_for_table(strTableName, m_field_primary_key);
+
   Box_DB_Table::init_db_details(strTableName);
 }
 
@@ -154,7 +158,7 @@ void Box_Data_Details::fill_from_database_layout()
       m_FlowTable.add_layout_group(iter->second);
     }
   }
-      
+
 }
 
 void Box_Data_Details::fill_from_database()
@@ -169,33 +173,37 @@ void Box_Data_Details::fill_from_database()
 
 
     Box_DB_Table::fill_from_database();
-         
-    type_vecFields listFieldsToShow = get_fields_to_show();
-    m_Fields =  listFieldsToShow;
-    if(!listFieldsToShow.empty())
+
+    m_Fields = get_fields_to_show();
+    type_vecFields fieldsToGet = m_Fields;
+
+    if(!fieldsToGet.empty())
     {
       fill_from_database_layout(); //TODO: Only do this when the layout has changed.
 
-      Field field_primary_key;
-      bool primary_key_found  = get_field_primary_key(field_primary_key);
+      //Add extra possibly-non-visible columns that we need:
+      fieldsToGet.push_back(m_field_primary_key); //Actually listFieldsToGet.
 
-      if(primary_key_found && !GlomConversions::value_is_empty(m_primary_key_value)) //If there is a record to show:
+      //g_warning("primary_key name = %s", m_field_primary_key.get_name().c_str());
+      const int index_primary_key = fieldsToGet.size() - 1;
+
+      if(!GlomConversions::value_is_empty(m_primary_key_value)) //If there is a record to show:
       {
         if(sharedconnection)
         {
           Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
           Glib::ustring sql_part_fields;
-          for(type_vecFields::const_iterator iter =  listFieldsToShow.begin(); iter != listFieldsToShow.end(); ++iter)
+          for(type_vecFields::const_iterator iter =  fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
           {
-            if(iter != listFieldsToShow.begin())
+            if(iter != fieldsToGet.begin())
               sql_part_fields += ",";
 
             sql_part_fields += iter->get_name();
           }
 
           std::stringstream query;
-          query << "SELECT " << sql_part_fields << " FROM " << m_strTableName << " WHERE " << get_table_name() + "." + get_primarykey_name() << " = " << field_primary_key.sql(m_primary_key_value);
+          query << "SELECT " << sql_part_fields << " FROM " << m_strTableName << " WHERE " << get_table_name() + "." + m_field_primary_key.get_name() << " = " << m_field_primary_key.sql(m_primary_key_value);
           Glib::RefPtr<Gnome::Gda::DataModel> result = connection->execute_single_command(query.str());
 
           if(result && result->get_n_rows())
@@ -208,9 +216,15 @@ void Box_Data_Details::fill_from_database()
 
               const int row_number = 0; //The only row.
               const int cols_count = result->get_n_columns();
+
+              //Get special possibly-non-visible field values:
+              if(index_primary_key < cols_count)
+                m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+
+              //Get field values to show:
               for(int i = 0; i < cols_count; ++i)
               {
-                const Field& field = m_Fields[i];
+                const Field& field = fieldsToGet[i];
 
                 //Field value:
                 Gnome::Gda::Value value = result->get_value_at(i, row_number);
@@ -225,7 +239,7 @@ void Box_Data_Details::fill_from_database()
         //Show blank record:
         //TODO
       }
-    } //if(!listFieldsToShow.empty())
+    } //if(!fieldsToGet.empty())
 
     //fill_related();
 
@@ -329,32 +343,27 @@ void Box_Data_Details::on_button_new()
 {
   if(confirm_discard_unstored_data())
   {
-    Field field_primary_key;
-    bool primary_key_found  = get_field_primary_key(field_primary_key);
-    if(primary_key_found)
+    Gnome::Gda::FieldAttributes fieldinfo = m_field_primary_key.get_field_info();
+    if(fieldinfo.get_auto_increment()) //If the primary key is an auto-increment:
     {
-      Gnome::Gda::FieldAttributes fieldinfo = field_primary_key.get_field_info();
-      if(fieldinfo.get_auto_increment()) //If the primary key is an auto-increment:
-      {
-        //Just make a new record, and show it:
-        guint generated_id = generate_next_auto_increment(m_strTableName, field_primary_key.get_name()); //TODO: This should return a Gda::Value
+      //Just make a new record, and show it:
+      guint generated_id = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
 
-        //TODO: Create the Gda::Value directly from the integer.
-        bool parsed = false; 
-        Gnome::Gda::Value primary_key_value = GlomConversions::parse_value(field_primary_key.get_glom_type(), util_string_from_decimal(generated_id), parsed);
-        if(parsed)
-        {
-          record_new(false /* use entered field data */, primary_key_value);
-          refresh_db_details(primary_key_value);
-        }
-      }
-      else
+      //TODO: Create the Gda::Value directly from the integer.
+      bool parsed = false; 
+      Gnome::Gda::Value primary_key_value = GlomConversions::parse_value(m_field_primary_key.get_glom_type(), util_string_from_decimal(generated_id), parsed);
+      if(parsed)
       {
-        //It's not an auto-increment primary key,
-        //so just blank the fields ready for a primary key later.
-        refresh_db_details_blank(); //shows blank record.
+        record_new(false /* use entered field data */, primary_key_value);
+        refresh_db_details(primary_key_value);
       }
     }
+    else
+    {
+      //It's not an auto-increment primary key,
+      //so just blank the fields ready for a primary key later.
+      refresh_db_details_blank(); //shows blank record.
+    }
 
   } //if(confirm_discard_unstored_data())
 }
@@ -495,111 +504,119 @@ Box_Data_Details::type_signal_user_requested_related_details Box_Data_Details::s
 }
 */
 
+void Box_Data_Details::on_flowtable_layout_changed()
+{
+  //Get new layout:
+  Document_Glom::type_mapLayoutGroupSequence layout_groups;
+  m_FlowTable.get_layout_groups(layout_groups);
+
+  Document_Glom* document = get_document();
+  if(document)
+    document->set_data_layout_groups("details", m_strTableName, layout_groups);
+
+  //Build the view again from the new layout:
+  fill_from_database_layout();
+
+  //And fill it with data:
+  fill_from_database();
+}
+
 void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
     return;
-    
-   const Glib::ustring strPrimaryKey_Name = get_primarykey_name();
-   const Glib::ustring strFieldName = id;
 
-   if(!strPrimaryKey_Name.empty())
-   {
-     Field fieldInfoPK;
-     bool test = get_field(strPrimaryKey_Name, fieldInfoPK);
-     if(!test)
-       g_warning("Box_Data_Details::on_flowtable_field_edited(): field not found");
+  const Glib::ustring strFieldName = id;
 
-     Gnome::Gda::Value primary_key_value = get_primary_key_value();
-     if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
-     {
-       Field field;
-       bool test = get_field(strFieldName, field);
-       if(!test)
-         g_warning("Box_Data_Details::on_flowtable_field_edited: field not found");
+  Gnome::Gda::Value primary_key_value = get_primary_key_value();
+  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
+  {
+    Field field;
+    bool test = get_field(strFieldName, field);
+    if(!test)
+      g_warning("Box_Data_Details::on_flowtable_field_edited: field not found");
 
-       //Update the field in the record (the record with this primary key):
-       try
-       {
-         Glib::ustring strQuery = "UPDATE " + m_strTableName;
-         strQuery += " SET " +  /* get_table_name() + "." +*/ strFieldName + " = " + field.sql(field_value);
-         strQuery += " WHERE " + get_table_name() + "." + strPrimaryKey_Name + " = " + fieldInfoPK.sql(get_primary_key_value());
-         bool bTest = Query_execute(strQuery);
+    //Update the field in the record (the record with this primary key):
+    try
+    {
+      Glib::ustring strQuery = "UPDATE " + m_strTableName;
+      strQuery += " SET " +  /* get_table_name() + "." +*/ strFieldName + " = " + field.sql(field_value);
+      strQuery += " WHERE " + get_table_name() + "." + m_field_primary_key.get_name() + " = " + m_field_primary_key.sql(get_primary_key_value());
+      bool bTest = Query_execute(strQuery);
 
-         if(!bTest)
-         {
-           //Update failed.
-           fill_from_database(); //Replace with correct values.
-         }
-         else
-         {
-           //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
-           m_FlowTable.set_field_value(field, field_value);
-           
-           //Get-and-set values for lookup fields, if this field triggers those relationships:
-           do_lookups(field, field_value, fieldInfoPK, primary_key_value);
+      if(!bTest)
+      {
+        //Update failed.
+        fill_from_database(); //Replace with correct values.
+      }
+      else
+      {
+        //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
+        m_FlowTable.set_field_value(field, field_value);
 
-           //If this is a foreign key then refresh the related records:
-           /*
-           bool bIsForeignKey = false;
-           Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
-           for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
-           {
-             const Relationship& relationship = *iter;
+        //Get-and-set values for lookup fields, if this field triggers those relationships:
+        do_lookups(field, field_value, m_field_primary_key, primary_key_value);
 
-             if(relationship.get_from_field() == strFieldName)
-             {
-               bIsForeignKey = true;
-               break;
-             }
-           }
+        //If this is a foreign key then refresh the related records:
+        /*
+        bool bIsForeignKey = false;
+        Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
+        for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
+        {
+          const Relationship& relationship = *iter;
 
-           if(bIsForeignKey)
-             fill_related();
-           */
-         }
+          if(relationship.get_from_field() == strFieldName)
+          {
+            bIsForeignKey = true;
+            break;
+          }
+        }
 
-       }
-       catch(const std::exception& ex)
-       {
-         handle_error(ex);
-       }
-     }
-     else
-     {
-       //There is no current primary key:
+        if(bIsForeignKey)
+          fill_related();
+        */
+      }
 
-       if(fieldInfoPK.get_field_info().get_auto_increment()) //If the primary key is an auto-increment:
-       {
-         if(strFieldName == strPrimaryKey_Name) //If edited field is the primary key.
-         {
-           //Warn user that they can't choose their own primary key:
-           Gtk::MessageDialog dialog ("The primary key is auto-incremented.\n You may not enter your own primary key value.");
-           dialog.run();
-         }
-       }
-       else
-       {
-         //It is not auto-generated:
+    }
+    catch(const std::exception& ex)
+    {
+      handle_error(ex);
+    }
+  }
+  else
+  {
+    //There is no current primary key value:
 
-         if(strFieldName == strPrimaryKey_Name) //if it is the primary key that' being edited.
-         {
-           //Create new record with this primary key,
-           //and all the other field values too.
-           //see comments after 'else':
-           record_new(true /* use entered field data */);
-         }
-         else
-         {
-           //The record does not exist yet.
-           //The values in the other fields will have to wait
-           //until the primary key is set by the user.
+    if(m_field_primary_key.get_field_info().get_auto_increment()) //If the primary key is an auto-increment:
+    {
+      if(strFieldName == m_field_primary_key.get_name()) //If edited field is the primary key.
+      {
+        //Warn user that they can't choose their own primary key:
+        Gtk::MessageDialog dialog ("The primary key is auto-incremented.\n You may not enter your own primary key value.");
+        dialog.run();
+      }
+    }
+    else
+    {
+      //It is not auto-generated:
 
-           set_unstored_data(true); //Cause a warning if this is never put into the database.
-         }
-       }
-     } //if(get_primary_key_value().size())
-   } //if(strPrimaryKey_Name.size())
+      if(strFieldName == m_field_primary_key.get_name()) //if it is the primary key that' being edited.
+      {
+        //Create new record with this primary key,
+        //and all the other field values too.
+        //see comments after 'else':
+        record_new(true /* use entered field data */);
+      }
+      else
+      {
+        //The record does not exist yet.
+        //The values in the other fields will have to wait
+        //until the primary key is set by the user.
+
+        set_unstored_data(true); //Cause a warning if this is never put into the database.
+      }
+    }
+  } //if(get_primary_key_value().size())
 }
 
 void Box_Data_Details::do_lookups(const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
@@ -640,6 +657,11 @@ void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
   m_FlowTable.set_design_mode( user_level == AppState::USERLEVEL_DEVELOPER );
 }
 
+bool Box_Data_Details::get_field_primary_key(Field& field) const
+{
+  field = m_field_primary_key;
+  return false;
+}
 
 
 
