@@ -66,10 +66,10 @@ void Box_DB_Table_Definition::init()
   m_colPrimaryKey = m_AddDel.add_column("Primary Key", AddDelColumnInfo::STYLE_Boolean);
 
   //Connect signals:
-  m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_AddDel_add));
-  m_AddDel.signal_user_requested_delete().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_AddDel_delete));
-  m_AddDel.signal_user_changed().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_AddDel_changed));
-  m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_AddDel_edit));
+  m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_adddel_add));
+  m_AddDel.signal_user_requested_delete().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_adddel_delete));
+  m_AddDel.signal_user_changed().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_adddel_changed));
+  m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_adddel_edit));
 
   //React to changes in the field properties:
   m_pDialog->signal_apply().connect(sigc::mem_fun(*this, &Box_DB_Table_Definition::on_Properties_apply));
@@ -93,7 +93,7 @@ void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& ite
   //Type:
   Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(fieldinfo.get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
 
-  Glib::ustring strType = Field::get_type_name( fieldType );
+  Glib::ustring strType = Field::get_type_name_ui( fieldType );
   m_AddDel.set_value(iter, m_colType, strType);
 
   //Unique:
@@ -115,7 +115,7 @@ void Box_DB_Table_Definition::fill_from_database()
     //Fields:
     m_AddDel.remove_all();
 
-    Field::type_map_type_names mapFieldTypes = Field::get_type_names();
+    Field::type_map_type_names mapFieldTypes = Field::get_type_names_ui();
 
     for(type_vecFields::iterator iter = m_Fields.begin(); iter != m_Fields.end(); iter++)
     {       
@@ -134,7 +134,7 @@ void Box_DB_Table_Definition::fill_from_database()
   fill_end();
 }
 
-void Box_DB_Table_Definition::on_AddDel_add(const Gtk::TreeModel::iterator& row)
+void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
 {
   Glib::ustring strName = m_AddDel.get_value(row, m_colName);
   if(!strName.empty())
@@ -165,7 +165,7 @@ void Box_DB_Table_Definition::on_AddDel_add(const Gtk::TreeModel::iterator& row)
   }
 }
 
-void Box_DB_Table_Definition::on_AddDel_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator&  rowEnd)
+void Box_DB_Table_Definition::on_adddel_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator&  rowEnd)
 {
   Gtk::TreeModel::iterator iterAfterEnd = rowEnd;
   if(iterAfterEnd != m_AddDel.get_model()->children().end())
@@ -184,7 +184,7 @@ void Box_DB_Table_Definition::on_AddDel_delete(const Gtk::TreeModel::iterator& r
   fill_from_database();
 }
 
-void Box_DB_Table_Definition::on_AddDel_changed(const Gtk::TreeModel::iterator& row, guint /* col */)
+void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator& row, guint /* col */)
 {
   //Get old field definition:
   Document_Glom* pDoc = static_cast<Document_Glom*>(get_document());
@@ -222,7 +222,7 @@ void Box_DB_Table_Definition::on_AddDel_changed(const Gtk::TreeModel::iterator&
   }
 }
 
-void Box_DB_Table_Definition::on_AddDel_edit(const Gtk::TreeModel::iterator& row)
+void Box_DB_Table_Definition::on_adddel_edit(const Gtk::TreeModel::iterator& row)
 {
   m_Field_BeingEdited = get_field_definition(row);
   
@@ -281,7 +281,7 @@ Field Box_DB_Table_Definition::get_field_definition(const Gtk::TreeModel::iterat
     //Type:
     const Glib::ustring& strType = m_AddDel.get_value(row, m_colType);
 
-    Field::glom_field_type glom_type =  Field::get_type_for_name(strType);
+    Field::glom_field_type glom_type =  Field::get_type_for_ui_name(strType);
     Gnome::Gda::ValueType fieldType = Field::get_gda_type_for_glom_type(glom_type);
 
     //Unique:
@@ -388,7 +388,9 @@ void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field fie
 
     //Update field names where they are used in relationships.
     if(field_name_old != field.get_name())
+    {
       pDoc->change_field_name(m_strTableName, field_name_old, field.get_name());
+    }
   }
 
 
@@ -479,6 +481,11 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_ol
         Glib::ustring conversion_command;
         switch(field.get_glom_type())
         {
+          case Field::TYPE_BOOLEAN: //CAST does not work if the destination type is numeric.
+          {
+            conversion_command = "FALSE"; //TODO: Find a way to convert: "to_number( " + field_old.get_name() + ", '999999999.99' )";
+            break;
+          }
           case Field::TYPE_NUMERIC: //CAST does not work if the destination type is numeric.
           {
             conversion_command = "to_number( " + field_old.get_name() + ", '999999999.99' )";
